declare type FoldMapper<R, T> = {
    none?: () => Resolvable<T>;
    loading?: () => Resolvable<T>;
    success?: (r: R) => Resolvable<T>;
    failed?: (e: any) => Resolvable<T>;
};
declare type WhenMapper<R, T> = {
    none?: () => Resolvable<T>;
    pending?: () => Resolvable<T>;
    resolved?: (r: R) => Resolvable<T>;
    rejected?: (e: any) => Resolvable<T>;
};
export declare enum Status {
    resolved = "resolved",
    rejected = "rejected",
    pending = "pending",
    none = "none"
}
declare type Resolvable<T> = T | Progress<T>;
interface PendingData {
    status: Status.pending;
}
interface NoneData {
    status: 'none';
}
export declare abstract class Progress<R> {
    static resolve: <R_1>(r: Resolvable<R_1>) => Progress<R_1>;
    static reject: (error: any) => Progress<any>;
    static pending: Progress<any>;
    static none: Progress<any>;
    static all: (...targets: Progress<any>[]) => Progress<any>;
    static race: <T>(...targets: Progress<T>[]) => Progress<T>;
    abstract readonly status: Status;
    readonly value?: R;
    readonly error?: any;
    map<T>(mapper: (r: R) => T): Progress<T>;
    flatMap<T>(mapper: (r: R) => T): Progress<T>;
    static readonly inProgress: Progress<any>;
    static success: <R_1>(r: Resolvable<R_1>) => Progress<R_1>;
    static fail: (error: any) => Progress<any>;
    readonly success: boolean;
    readonly inProgress: boolean;
    readonly failed: boolean;
    readonly result: R | undefined;
    readonly isCompleted: boolean;
    unwrap(): R;
    ifSuccess<T>(mapper: (r: R) => Resolvable<T>): T | null;
    fold<T>(mapper: FoldMapper<R, T>): T | null;
    readonly isNone: boolean;
    readonly isPending: boolean;
    readonly isResolved: boolean;
    readonly isRejected: boolean;
    readonly isFulfilled: boolean;
    ifNone<T>(supplier?: () => Resolvable<T>): Progress<T>;
    ifPending<T>(supplier?: () => Resolvable<T>): Progress<T>;
    then<T>(): Progress<T>;
    then<T>(mapper?: (r: R) => Resolvable<T>): Progress<T>;
    then<T1, T2>(mapper?: (r: R) => Resolvable<T1>, errorMapper?: (e: any) => Resolvable<T2>): Progress<T1 | T2>;
    catch<T>(errorMapper?: (e: any) => Resolvable<T>): Progress<T>;
    finally<T>(supplier?: () => Resolvable<T>): Progress<T>;
    abstract when<T>(mapper: WhenMapper<R, T>): Progress<T>;
    get(): R;
    equals(progress: Progress<any>): boolean;
}
interface ResolvedData<R> {
    status: Status.resolved;
    value: R;
}
interface RejectedData {
    status: Status.rejected;
    error: any;
}
export declare type Data<R> = NoneData | PendingData | ResolvedData<R> | RejectedData;
export declare const from: <R>(serialized: PendingData | NoneData | RejectedData | Progress<R> | ResolvedData<R>) => Progress<R>;
export declare const thunk: <R>(type: string, promiseOrFunc: Promise<R> | (() => Promise<R>), extras?: any) => (dispatch: (a: any) => void) => Promise<any>;
export declare const thunkProgress: <R>(type: string, promiseOrFunc: Promise<R> | (() => Promise<R>), extras?: any) => (dispatch: (a: any) => void) => Promise<any>;
export default Progress;
export declare const inProgress: Progress<any>, success: <R>(r: Resolvable<R>) => Progress<R>, fail: (error: any) => Progress<any>;
export declare const none: Progress<any>, pending: Progress<any>, resolve: <R>(r: Resolvable<R>) => Progress<R>, reject: (error: any) => Progress<any>, all: (...targets: Progress<any>[]) => Progress<any>, race: <T>(...targets: Progress<T>[]) => Progress<T>;
